@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mms.h,v 1.10 2005/12/12 09:24:19 theuraeus Exp $
+ * $Id: mms.h,v 1.11 2006/06/06 09:49:15 shawarma Exp $
  *
  * libmms public header
  */
@@ -41,17 +41,25 @@
 #include "bswap.h"
 #include "mmsio.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 typedef struct mms_s mms_t;
 
 mms_t*   mms_connect (mms_io_t *io, void *data, const char *url, int bandwidth);
 
-int      mms_read (mms_io_t *io, mms_t *this, char *data, int len);
-uint32_t mms_get_length (mms_t *this);
-void     mms_close (mms_t *this);
+int      mms_read (mms_io_t *io, mms_t *instance, char *data, int len);
+uint32_t mms_get_length (mms_t *instance);
+void     mms_close (mms_t *instance);
 
-int      mms_peek_header (mms_t *this, char *data, int maxsize);
+int      mms_peek_header (mms_t *instance, char *data, int maxsize);
 
-off_t    mms_get_current_pos (mms_t *this);
+off_t    mms_get_current_pos (mms_t *instance);
+
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
 
 #endif
 
