@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mmsh.h,v 1.2 2005/12/12 09:24:20 theuraeus Exp $
+ * $Id: mmsh.h,v 1.4 2006/06/06 09:52:28 shawarma Exp $
  *
  * libmmsh public header
  */
@@ -26,18 +26,28 @@
 #define HAVE_MMSH_H
 
 #include <inttypes.h>
+#include <stdio.h>
+#include <sys/types.h>
 #include "mmsio.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 typedef struct mmsh_s mmsh_t;
 
 char*    mmsh_connect_common(int *s ,int *port, char *url, char **host, char **path, char **file);
 mmsh_t*   mmsh_connect (mms_io_t *io, void *data, const char *url_, int bandwidth);
 
-int      mmsh_read (mms_io_t *io, mmsh_t *this, char *data, int len);
-uint32_t mmsh_get_length (mmsh_t *this);
-off_t    mmsh_get_current_pos (mmsh_t *this);
-void     mmsh_close (mmsh_t *this);
+int      mmsh_read (mms_io_t *io, mmsh_t *instance, char *data, int len);
+uint32_t mmsh_get_length (mmsh_t *instance);
+off_t    mmsh_get_current_pos (mmsh_t *instance);
+void     mmsh_close (mmsh_t *instance);
 
-int      mmsh_peek_header (mmsh_t *this, char *data, int maxsize);
+int      mmsh_peek_header (mmsh_t *instance, char *data, int maxsize);
+
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
 
 #endif
